#==============================================================#
# Analysis of the GDP.pw.rel distribution initial/final period #
# and map of GDP.pw.rel at regional level                      #  
#==============================================================#

#Remove all objects
rm(list=ls(all=TRUE))

load("AlldataDrivers_CE2016.RData")

library(foreach)
library(doMC)
library(base)
registerDoMC(1)

# Functions used in the estimation #
####################################
#------------------------------------------------------#
# Estimation of univariate density via Adaptive Kernel #
source("univariateDensityAdaptive_Parallel.r")
#--------------------------------------------------------------#
# Calculation of the weighted standard deviation of a variable #
source("weighted.sd.r")
#----------------------------------#
# Estimation of transitions matrix #
source("tranMatrixWeightedObs.r")
#-----------------------------------------------------------------------------------------------------------#
# Estimation of kernel density via Adaptive Kernel with different variables' range and ergodic distribution #
source("bivariateDensityKernelAdaptiveDifferentRanges_Matrix_Parallel.r")
#-----------------------------------------------------------------------------------------------------------#
# Bootstrap estimation of confidence bands for the median of the kernel density and the ergodic distribution#
source("median.bootstrap.conf.bandDifferentRanges_Matrix_Parallel.r")
#-----------------------------------------#
# Calculation of the value of an integral #
source("numericalIntegration.r")


#Define GDP per worker
GDPpw <- GDPpw.1991.2014.final
#Define GDP per worker in Relative terms (w.r.t the sample average in each year)
GDPpwRel <- GDPpwRel.1991.2014.final

numYears <- ncol(GDPpwRel)

#Firtst and last year of the sample
PROD.InitialYear.REL <- (GDPpwRel[,1])  
PROD.FinalYear.REL <-  (GDPpwRel[,(numYears)])

#=========================================================#
# Density distributions for GDP PER WORKER relative terms #
#=========================================================#

yRel1991 <- GDPpwRel[,colnames(GDPpwRel)=="1991"]
yRel1995 <- GDPpwRel[,colnames(GDPpwRel)=="1993"]
yRel2000 <- GDPpwRel[,colnames(GDPpwRel)=="2000"]
yRel2005 <- GDPpwRel[,colnames(GDPpwRel)=="2005"]
yRel2008 <- GDPpwRel[,colnames(GDPpwRel)=="2008"]
yRel2012 <- GDPpwRel[,colnames(GDPpwRel)=="2012"]
yRel2014 <- GDPpwRel[,colnames(GDPpwRel)=="2014"]

yRel1991Dens <- univariateDensityAdaptive_Parallel(yRel1991, figure=FALSE)
yRel1995Dens <- univariateDensityAdaptive_Parallel(yRel1995, figure=FALSE)
yRel2000Dens <- univariateDensityAdaptive_Parallel(yRel2000, figure=FALSE)
yRel2005Dens <- univariateDensityAdaptive_Parallel(yRel2005, figure=FALSE)
yRel2008Dens <- univariateDensityAdaptive_Parallel(yRel2008, figure=FALSE)
yRel2012Dens <- univariateDensityAdaptive_Parallel(yRel2012, figure=FALSE)
yRel2014Dens <- univariateDensityAdaptive_Parallel(yRel2014, figure=FALSE)

#require(graphics)
colorForGraph <- palette(rainbow(7, start = 0.7, end=0.1))

#dev.new()
plot(yRel1991Dens$eval.points, yRel1991Dens$estimate,type="l",xlab="Relative per worker GDP", ylab="Density", xlim=c(0,2.25) , ylim=c(0,1.6), lwd=2, col=colorForGraph[1])      
lines(yRel1995Dens$eval.points, yRel1995Dens$estimate,lwd=2, col=colorForGraph[2])         
lines(yRel2000Dens$eval.points, yRel2000Dens$estimate,lwd=2, col=colorForGraph[3])         
lines(yRel2005Dens$eval.points, yRel2005Dens$estimate,lwd=2, col=colorForGraph[4])         
lines(yRel2008Dens$eval.points, yRel2008Dens$estimate,lwd=2, col=colorForGraph[5])         
lines(yRel2012Dens$eval.points, yRel2012Dens$estimate,lwd=2.5, col=colorForGraph[6])
lines(yRel2014Dens$eval.points, yRel2014Dens$estimate,lwd=2.5, col=colorForGraph[7])
legend("topright",c("1991", "1995", "2000", "2005", "2008", "2012", "2014"),col =colorForGraph,text.col= "black",lwd=c(2,2,2,2),  merge = TRUE, y.intersp = 1, x.intersp = 0.5,bg="lightgray")

grid(nx=10,ny=10)



#####################
# Transition matrix #
#####################

#dev.new()
plot(yRel1991Dens$eval.points, yRel1991Dens$estimate,type="l",xlab="Relative per worker GDP", ylab="Density", xlim=c(0,2.25) , ylim=c(0,1.6), lwd=2, col="black")      
lines(yRel2014Dens$eval.points, yRel2014Dens$estimate,lwd=2.5, col="blue")
legend("topright",c("1991", "2014"),col =c("black", "blue"),text.col= "black",lwd=c(2,2),  merge = TRUE, y.intersp = 1, x.intersp = 0.5,bg="lightgray")
grid(nx=10,ny=10)

#Matrix of actual and future observations
numLag <- 10
#Vectors with the actual and future observation with lag equal to numLag
ini <- 1
fin <- ncol(GDPpw)
PROD.InitialYears.REL <- GDPpwRel[,ini:(fin-numLag)]
PROD.FinalYears.REL <- GDPpwRel[,(ini+numLag):fin]
www <- !is.na(PROD.InitialYears.REL) & !is.na(PROD.FinalYears.REL)

futObs <- c(PROD.FinalYears.REL[www])
actObs <- c(PROD.InitialYears.REL[www])

#Define the quantiles to get the classes of Markov matrix
kk <- c(actObs,futObs )
stateAct <- as.vector(c(quantile(kk, probs=seq(0,1,0.2))))

#Plot distributions and classes
plot(yRel1991Dens$eval.points, yRel1991Dens$estimate,type="l",xlab="Relative per worker GDP", ylab="Density", xlim=c(0,2.25) , ylim=c(0,1.6), lwd=2, col="black")      
lines(yRel2014Dens$eval.points, yRel2014Dens$estimate,lwd=2.5, col="blue")
legend("topright",c("1991", "2014"),col =c("black", "blue"),text.col= "black",lwd=c(2,2),  merge = TRUE, y.intersp = 1, x.intersp = 0.5,bg="lightgray")
grid(nx=10,ny=10)
abline(v=stateAct[2])
abline(v=stateAct[3])
abline(v=stateAct[4])
abline(v=stateAct[5])

#Define weight of observations (here is 1)
wei <- vector("numeric",length=length(actObs))+1
numStates <- length(stateAct)-1

#Estimate the Markov and Transition matrix
estMatrix <- tranMatrixWeightedObs(act=actObs,fut=futObs,statesFut=stateAct,
                                   statesAct=stateAct,numStates=numStates, wei=wei)
print(estMatrix)                         

#Classes uses to define the state
state <- round(as.matrix(t(stateAct)), digits=2)
spaceStates <- rbind(c("1", "2", "3", "4", "5"), 
     c(paste(state[1],state[2], sep="-"), 
      paste(state[2],state[3], sep="-"), 
      paste(state[3],state[4], sep="-"),
      paste(state[4],state[5], sep="-"),
      paste(state[5],state[6], sep="-"))
      )
rownames(spaceStates) <- c("State", "Range")
print(spaceStates)

#Estimated Markovian matrix
MarkovMatrix <- estMatrix$markovM
colnames(MarkovMatrix) <- c("1", "2", "3", "4", "5")
rownames(MarkovMatrix) <- c("1", "2", "3", "4", "5")
print(MarkovMatrix)

#Estimated Transition matrix
transitionMatrix <- round(estMatrix$transM, digits=2)
colnames(transitionMatrix) <- c("1", "2", "3", "4", "5")
rownames(transitionMatrix) <- c("1", "2", "3", "4", "5")
print(transitionMatrix)

  

#**********************************#
# Change of state space definition #
#**********************************#
stateAct <- c(0,0.6,0.75,1,1.35,2.5)

#Plot distributions and classes
plot(yRel1991Dens$eval.points, yRel1991Dens$estimate,type="l",xlab="Relative per worker GDP", ylab="Density", xlim=c(0,2.25) , ylim=c(0,1.6), lwd=2, col="black")      
lines(yRel2014Dens$eval.points, yRel2014Dens$estimate,lwd=2.5, col="blue")
legend("topright",c("1991", "2014"),col =c("black", "blue"),text.col= "black",lwd=c(2,2),  merge = TRUE, y.intersp = 1, x.intersp = 0.5,bg="lightgray")
grid(nx=10,ny=10)
abline(v=stateAct[2])
abline(v=stateAct[3])
abline(v=stateAct[4])
abline(v=stateAct[5])

#Estimate the Markov and Transition matrix
estMatrix <- tranMatrixWeightedObs(act=actObs,fut=futObs,statesFut=stateAct,
                                   statesAct=stateAct,numStates=numStates, wei=wei)
print(estMatrix)                         

#Classes uses to define the state
state <- round(as.matrix(t(stateAct)), digits=2)
spaceStates2 <- rbind(c("1", "2", "3", "4", "5"), 
                     c(paste(state[1],state[2], sep="-"), 
                       paste(state[2],state[3], sep="-"), 
                       paste(state[3],state[4], sep="-"),
                       paste(state[4],state[5], sep="-"),
                       paste(state[5],state[6], sep="-"))
)
rownames(spaceStates2) <- c("State", "Range")
print(spaceStates2)

#Estimated Markovian matrix
MarkovMatrix2 <- estMatrix$markovM
colnames(MarkovMatrix2) <- c("1", "2", "3", "4", "5")
rownames(MarkovMatrix2) <- c("1", "2", "3", "4", "5")
print(MarkovMatrix2)

#Estimated Transition matrix
transitionMatrix2 <- round(estMatrix$transM, digits=2)
colnames(transitionMatrix2) <- c("1", "2", "3", "4", "5")
rownames(transitionMatrix2) <- c("1", "2", "3", "4", "5")
print(transitionMatrix2)

#Estimated Ergodic distribution
ergD <- round(as.matrix(t(estMatrix$ergoDistr)), digits=2)
ergoDistribution2 <- rbind(c("1", "2", "3", "4", "5"), ergD)
rownames(ergoDistribution2) <- c("State", "Mass of probability")
print(ergoDistribution2)


#=============================================#
# Stochastic Kernel  of GDP pw Relative terms #
#=============================================#

#####################
# Stochastic Kernel #
#####################

##ATTENTION: the computational burder exponentially increases with 
#ngridPar and the number of bootstrasps
#ngridPar = 150 and B=100 is a good compromize (with 12 CPU about 2 hour)

#Set parameters for estimation
#Range of observations for stochastic kernel
rangeOBS <- c(-0.25,2.5)
#Number of evaluation points for stochastic kernel  
ngridPar <- 150
# Alpha parameter of the bivariate and univariate adaptive kernel
alpha <- 0.5
alpha.uni.density <- 0.5
#Number of bootstraps for the median and the ergodic of stochastic kernel  
B <- 100
#Parameter for the plot
nlevelsPar <- 10

#Estimation of the Stochastic Kernel
t1 <- Sys.time()
stocKernelOBS <- bivariateDensityKernelAdaptiveDifferentRanges_Matrix_Parallel(x=cbind(futObs,actObs),
                                                                               rangeObs.x=rangeOBS,rangeObs.y=rangeOBS, ngrid=ngridPar,alpha=0.5,figure=FALSE,                                                                               ergodicDistr=TRUE,obsNormalized=TRUE)
print(Sys.time()- t1)

 # Load the result of previous estimate of stochastic kernel and bootstrap analysis
#load(file="estimatedUncondDistrDynamics_GDPpw_EU27_with10_year_lag.RData")



load("distribDynamics_GDPpw_1991_2014_5_year_lag.RData")

#-----------------------------#
# Plot the Stochastic kernel  #
#-----------------------------#
dev.new()
contour(stocKernelOBS$evalPoints.x,stocKernelOBS$evalPoints.x,
        t(stocKernelOBS$stocKernel),nlevels=15,
        xlim=c(0,2),ylim=c(0,2),
        labcex = 0.3,
        xlab="Relative GDP per worker (t)",
        ylab="Relative GDP per worker (t+5)",lwd=0.5)
grid(nx=14,ny=14)
#Plot the limits of classes
lines(c(0,4),c(0,4),lwd=2)
#Plot the median of conditional distribution

lines(stocKernelOBS$evalPoints.y,stocKernelOBS$averageKernel,lwd=4,col="black",lty=1)  
abline(h=1)
abline(v=1)

#----------------------------------------------------------------#
# Plot Density initial final years plus the ergodic distribution #
#----------------------------------------------------------------#
densityInitialYear <- univariateDensityAdaptive_Parallel(GDPpwRel[,1],figure=FALSE)
densityLastYear <-  univariateDensityAdaptive_Parallel(GDPpwRel[,ncol(GDPpwRel)],figure=FALSE)

#-----------------------------#
# Density initial final years #
#-----------------------------#
dev.new()
plot(densityLastYear$eval.points,densityLastYear$estimate,type="l",lwd=2,ylab="Density",
     xlab="Relative GDP per worker",xlim=c(0,2), ylim=c(0,1.6))  
lines(densityInitialYear$eval.points,densityInitialYear$estimate,lty=3,lwd=2)
grid(nx=14,ny=14)
legend(0.1,1.5,c("1991","2014"),lwd=c(2,2),lty=c(3,1),bg="grey", cex=0.8)

dev.new()
par(bg='transparent')
plot(stocKernelOBS$evalPointsNorm,stocKernelOBS$ergoDistrNorm,type="l",lwd=2,
     ylab="Density",
     xlab="Relative GDP per worker",xlim=c(0,2))  
lines(densityLastYear$eval.points,densityLastYear$estimate,lty=2,lwd=2)
lines(densityInitialYear$eval.points,densityInitialYear$estimate,lty=3,lwd=2)
grid(nx=10,ny=10)
legend("topright",c("1991","2014","Ergodic"),lwd=c(2,2,2),lty=c(3,2,1),bg="lightgray", 
       cex=0.9)
#dev.copy2eps(file="ergodicDistrOBS_GDPpw_1991_2014.eps")


